# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
#
# Copyright 2015 Andraz Vrhovec <andraz@vrhovec.si>
#
# This file is part of duplicity.
#
# Duplicity is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# Duplicity is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with duplicity; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import os

import duplicity.backend
from duplicity import log
from duplicity import util
from duplicity.errors import BackendException
import posixpath
import uuid

class KoofrBackend(duplicity.backend.Backend):
    """
    Backend for Koofr
    """
    def __init__(self, parsed_url):
        duplicity.backend.Backend.__init__(self, parsed_url)

        parts = parsed_url.path.lstrip('/').split('/')

        self.api_base = "https://app.koofr.net"
        self.mount_id = None

        start_index = 2

        try:
            uuid.UUID(parts[0])
            self.mount_id = parts[0]
            start_index = 1
        except ValueError:
            self.api_base = "https://" + parts[0]

        if self.mount_id is None:
            try:
                uuid.UUID(parts[1])
                self.mount_id = parts[1]
            except ValueError:
                raise BackendException("Malformed mount ID. Should be in UUID4 format")


        self.dir = '/' + posixpath.join(*parts[start_index:]).rstrip('/')

        try:
            import koofr
            self.client = koofr.client.KoofrClient(self.api_base)
        except ImportError:
            raise BackendException("This backend requires "
                                   "the python-koofr library. sudo pip install koofr")

        try:
            self.client.set_token(os.environ["KOOFR_TOKEN"])
        except:
            try:
                self.client.authenticate(os.environ["KOOFR_USERNAME"], os.environ["KOOFR_PASSWORD"])
            except:
                raise BackendException("You must specify either KOOFR_TOKEN or KOOFR_USERNAME and KOOFR_PASSWORD env variables")

        try:
            self.client.files_new_folder(self.mount_id, self.dir)
        except Exception:
            pass

    def _put(self, source_path, remote_filename):
        try:
            self.client.files_remove(self.mount_id, posixpath.join(self.dir, remote_filename))
        except:
            pass
        with open(source_path.name, "rb") as f:
            self.client.files_put(self.mount_id, posixpath.join(self.dir, remote_filename), f)

    def _get(self, remote_filename, local_path):
        self.client.files_get_to_path(self.mount_id, posixpath.join(self.dir, remote_filename), local_path.name)

    def _list(self):
        return [f["name"] for f in self.client.files_list(self.mount_id, self.dir)]

    def _delete(self, filename):
        self.client.files_remove(self.mount_id, posixpath.join(self.dir, filename))

    def _query(self, filename):
        info = self.client.files_info(self.mount_id, posixpath.join(self.dir, filename))
        return {'size': int(info["size"])}

duplicity.backend.register_backend("koofr", KoofrBackend)
